#include <string.h>
#include "gtk/gtk.h"
#include "gdome.h"
#include "gnome-xml/tree.h"
#include "gdome-xml.h"

typedef struct _Gdome_xml_Node Gdome_xml_Node;
typedef struct _Gdome_xml_Attr Gdome_xml_Attr;
typedef struct _Gdome_xml_CharacterData Gdome_xml_CharacterData;
typedef struct _Gdome_xml_Text Gdome_xml_Text;
typedef struct _Gdome_xml_CDATASection Gdome_xml_CDATASection;
typedef struct _Gdome_xml_Comment Gdome_xml_Comment;
typedef struct _Gdome_xml_DOMImplementation Gdome_xml_DOMImplementation;
typedef struct _Gdome_xml_Document Gdome_xml_Document;
typedef struct _Gdome_xml_DocumentFragment Gdome_xml_DocumentFragment;
typedef struct _Gdome_xml_DocumentType Gdome_xml_DocumentType;
typedef struct _Gdome_xml_Element Gdome_xml_Element;
typedef struct _Gdome_xml_Entity Gdome_xml_Entity;
typedef struct _Gdome_xml_EntityReference Gdome_xml_EntityReference;
typedef struct _Gdome_xml_NamedNodeMap Gdome_xml_NamedNodeMap;
typedef struct _Gdome_xml_el_NamedNodeMap Gdome_xml_el_NamedNodeMap;
typedef struct _Gdome_xml_NodeList Gdome_xml_NodeList;
typedef struct _Gdome_xml_Notation Gdome_xml_Notation;
typedef struct _Gdome_xml_Parser Gdome_xml_Parser;
typedef struct _Gdome_xml_ProcessingInstruction Gdome_xml_ProcessingInstruction;

struct _Gdome_xml_Node {
  GdomeNode super;
  xmlNode *n;
};

struct _Gdome_xml_Attr {
  GdomeAttr super;
  xmlAttr *a;
};

struct _Gdome_xml_CharacterData {
  GdomeCharacterData super;
  xmlNode *n;
};

struct _Gdome_xml_Text {
  GdomeText super;
  xmlNode *n;
};

struct _Gdome_xml_CDATASection {
  GdomeCDATASection super;
  xmlNode *n;
};

struct _Gdome_xml_Comment {
  GdomeComment super;
  xmlNode *n;
};

struct _Gdome_xml_DOMImplementation {
  GdomeDOMImplementation super;
};

struct _Gdome_xml_Document {
  GdomeDocument super;
  xmlDoc *doc;
};

struct _Gdome_xml_DocumentFragment {
  GdomeDocumentFragment super;
  xmlNode *n;
};

struct _Gdome_xml_DocumentType {
  GdomeDocumentType super;
  xmlDtd *dt;
};

struct _Gdome_xml_Element {
  GdomeElement super;
  xmlNode *n;
};

struct _Gdome_xml_Entity {
  GdomeEntity super;
  xmlNode *n;
};

struct _Gdome_xml_EntityReference {
  GdomeEntityReference super;
  xmlNode *n;
};

struct _Gdome_xml_NamedNodeMap {
  GdomeNamedNodeMap super;
};

/* A special NamedNodeMap for implementing the attributes method on Element */
struct _Gdome_xml_el_NamedNodeMap {
  GdomeNamedNodeMap super;
  xmlNode *n;
};

struct _Gdome_xml_NodeList {
  GdomeNodeList super;
};

struct _Gdome_xml_Notation {
  GdomeNotation super;
  xmlNotation *not;
};

struct _Gdome_xml_Parser {
  GdomeParser super;
};

struct _Gdome_xml_ProcessingInstruction {
  GdomeProcessingInstruction super;
  xmlNode *n;
};

const GdomeNodeVtab gdome_xml_n_vtab;
const GdomeAttrVtab gdome_xml_a_vtab;
const GdomeCharacterDataVtab gdome_xml_cd_vtab;
const GdomeTextVtab gdome_xml_t_vtab;
const GdomeCDATASectionVtab gdome_xml_cds_vtab;
const GdomeCommentVtab gdome_xml_c_vtab;
const GdomeDOMImplementationVtab gdome_xml_domi_vtab;
const GdomeDocumentVtab gdome_xml_doc_vtab;
const GdomeDocumentFragmentVtab gdome_xml_df_vtab;
const GdomeDocumentTypeVtab gdome_xml_dt_vtab;
const GdomeElementVtab gdome_xml_el_vtab;
const GdomeEntityVtab gdome_xml_ent_vtab;
const GdomeEntityReferenceVtab gdome_xml_er_vtab;
const GdomeNamedNodeMapVtab gdome_xml_nnm_vtab;
const GdomeNamedNodeMapVtab gdome_xml_el_nnm_vtab;
const GdomeNodeListVtab gdome_xml_nl_vtab;
const GdomeNotationVtab gdome_xml_not_vtab;
const GdomeParserVtab gdome_xml_pars_vtab;
const GdomeProcessingInstructionVtab gdome_xml_pi_vtab;

/* indexed by node type */
const void * gdome_xml_vtabs[] = {
  NULL,
  &gdome_xml_el_vtab,
  &gdome_xml_a_vtab,
  &gdome_xml_t_vtab,
  &gdome_xml_cds_vtab,
  &gdome_xml_er_vtab,
  &gdome_xml_ent_vtab,
  &gdome_xml_pi_vtab,
  &gdome_xml_c_vtab,
  &gdome_xml_doc_vtab,
  &gdome_xml_dt_vtab,
  &gdome_xml_df_vtab,
  &gdome_xml_not_vtab
};

static GdomeNode *
gdome_xml_n_mkref (xmlNode *n)
{
  Gdome_xml_Node *result;

  if (n == NULL)
     return NULL;
  result = g_new (Gdome_xml_Node, 1);
  result->super.vtab = (GdomeNodeVtab *)gdome_xml_vtabs[n->type];
  result->n = n;
  return (GdomeNode *)result;
}

static GdomeDocument *
gdome_xml_doc_mkref (xmlDoc *doc)
{
  Gdome_xml_Document *result;

  if (doc == NULL)
     return NULL;
  result = g_new (Gdome_xml_Document, 1);
  result->super.vtab = (GdomeDocumentVtab *)&gdome_xml_doc_vtab;
  result->doc = doc;
  return (GdomeDocument *)result;
}

void
gdome_xml_str_unref1 (GdomeDOMString *self)
{
  g_free (self);
}

GdomeDOMString *
gdome_xml_str_mkref (char *str)
{
  GdomeDOMString *ds;

  if (str == NULL)
    return NULL;
  ds = g_new (GdomeDOMString, 1);
  ds->unref = gdome_xml_str_unref1;
  ds->str = str;
  return ds;
}

/* also do a string dup here */

/* implementations of methods in interface Node */

static void
gdome_xml_n_unref (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  g_free (self);
}

static GdomeDOMString *
gdome_xml_n_nodeName (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;
  char *str;

  switch (n->type)
    {
    case XML_ELEMENT_NODE:
      str = n->name;
      break;
    case XML_TEXT_NODE:
      str = "#text";
      break;
    case XML_CDATA_SECTION_NODE:
      str = "#cdata-section";
      break;
    case XML_ENTITY_REF_NODE:
      str = NULL; /* todo - I don't think gnome-xml actually pushes
		     entity references into the tree */
      break;
    case XML_ENTITY_NODE:
      str = NULL; /* todo - I don't think gnome-xml actually pushes
		     entities into the tree */
      break;
    case XML_PI_NODE:
      str = NULL; /* todo - I don't think gnome-xml actually pushes
		     processing instructions into the tree */
      break;
    case XML_COMMENT_NODE:
      str = "#comment";
      break;
    case XML_DOCUMENT_FRAG_NODE:
      str = "#document-fragment";
      break;
    case XML_NOTATION_NODE:
      str = NULL; /* todo - I don't think gnome-xml actually pushes
		     notations into the tree */
      break;
    default:
      /* some of the others, like Attr and DocumentType, can't happen
	 here because they have their own method implementations */
      str = NULL;
      break;
    }

  return gdome_xml_str_mkref (str);
}

static GdomeDOMString *
gdome_xml_n_nodeValue (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  char *str;

  switch (n->type)
    {
    case XML_ELEMENT_NODE:
      str = n->name;
      break;
    case XML_TEXT_NODE:
      str = n->content;
      break;
    case XML_CDATA_SECTION_NODE:
      str = n->content;
      break;
    case XML_PI_NODE:
      str = NULL; /* todo - I don't think gnome-xml actually pushes
		     processing instructions into the tree */
      break;
    case XML_COMMENT_NODE:
      str = NULL; /* todo - I don't think gnome-xml actually pushes
		     comments into the tree */
      break;
    default:
      /* some of the others, like Attr and DocumentType, can't happen
	 here because they have their own method implementations */
      str = NULL;
      break;
    }

  return gdome_xml_str_mkref (str);
}

static void
gdome_xml_n_set_nodeValue (GdomeNode *self, GdomeDOMString *nodeValue)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

}

static unsigned short
gdome_xml_n_nodeType (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return n->type;
}

static GdomeNode *
gdome_xml_n_parentNode (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return gdome_xml_n_mkref (n->parent);
}

static GdomeNodeList *
gdome_xml_n_childNodes (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeNode *
gdome_xml_n_firstChild (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return gdome_xml_n_mkref (n->childs);
}

static GdomeNode *
gdome_xml_n_lastChild (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return gdome_xml_n_mkref (n->last);
}

static GdomeNode *
gdome_xml_n_previousSibling (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return gdome_xml_n_mkref (n->prev);
}

static GdomeNode *
gdome_xml_n_nextSibling (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return gdome_xml_n_mkref (n->next);
}

static GdomeNamedNodeMap *
gdome_xml_n_attributes (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;
  Gdome_xml_el_NamedNodeMap *result;

  if (n->type != GDOME_ELEMENT_NODE)
    return NULL;
  result = g_new (Gdome_xml_el_NamedNodeMap, 1);
  result->super.vtab = &gdome_xml_el_nnm_vtab;
  result->n = n;
  return (GdomeNamedNodeMap *)result;
}

static GdomeDocument *
gdome_xml_n_ownerDocument (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return gdome_xml_doc_mkref (n->doc);
}

static GdomeNode *
gdome_xml_n_insertBefore (GdomeNode *self, GdomeNode *newChild, GdomeNode *refChild)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeNode *
gdome_xml_n_replaceChild (GdomeNode *self, GdomeNode *newChild, GdomeNode *oldChild)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeNode *
gdome_xml_n_removeChild (GdomeNode *self, GdomeNode *oldChild)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeNode *
gdome_xml_n_appendChild (GdomeNode *self, GdomeNode *newChild)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeBoolean
gdome_xml_n_hasChildNodes (GdomeNode *self)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return (n->childs != NULL);
}

static GdomeNode *
gdome_xml_n_cloneNode (GdomeNode *self, GdomeBoolean deep)
{
  Gdome_xml_Node *priv = (Gdome_xml_Node *)self;
  xmlNode *n = priv->n;

  return NULL;
}

/* implementations of methods in interface Attr */

static void
gdome_xml_a_unref (GdomeNode *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  g_free (self);
}

static GdomeDOMString *
gdome_xml_a_nodeName (GdomeNode *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return gdome_xml_str_mkref (a->name);
}

static GdomeDOMString *
gdome_xml_a_nodeValue (GdomeNode *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;
  xmlNode *val = a->val;

  /* todo: expand entity references; bleah! */
  return gdome_xml_str_mkref (val->content);
}

static void
gdome_xml_a_set_nodeValue (GdomeNode *self, GdomeDOMString *nodeValue)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;
  xmlNode *val = a->val;

  xmlNodeSetContent (val, nodeValue->str);
}

static unsigned short
gdome_xml_a_nodeType (GdomeNode *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return GDOME_ATTRIBUTE_NODE;
}

static GdomeNode *
gdome_xml_a_parentNode (GdomeNode *self)
{
  return NULL;
}

static GdomeNodeList *
gdome_xml_a_childNodes (GdomeNode *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return NULL;
}

static GdomeNode *
gdome_xml_a_firstChild (GdomeNode *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return gdome_xml_n_mkref (a->val);
}

static GdomeNode *
gdome_xml_a_lastChild (GdomeNode *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return NULL;
}

static GdomeNode *
gdome_xml_a_previousSibling (GdomeNode *self)
{
  return NULL;
}

static GdomeNode *
gdome_xml_a_nextSibling (GdomeNode *self)
{
  return NULL;
}

static GdomeNamedNodeMap *
gdome_xml_a_attributes (GdomeNode *self)
{
  return NULL;
}

static GdomeDocument *
gdome_xml_a_ownerDocument (GdomeNode *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return NULL; /* this is tricky - if it has a parent, then can get
		  document from parent. otherwise ??? */
}

static GdomeNode *
gdome_xml_a_insertBefore (GdomeNode *self, GdomeNode *newChild, GdomeNode *refChild)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return NULL;
}

static GdomeNode *
gdome_xml_a_replaceChild (GdomeNode *self, GdomeNode *newChild, GdomeNode *oldChild)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return NULL;
}

static GdomeNode *
gdome_xml_a_removeChild (GdomeNode *self, GdomeNode *oldChild)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return NULL;
}

static GdomeNode *
gdome_xml_a_appendChild (GdomeNode *self, GdomeNode *newChild)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return NULL;
}

static GdomeBoolean
gdome_xml_a_hasChildNodes (GdomeNode *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return 0;
}

static GdomeNode *
gdome_xml_a_cloneNode (GdomeNode *self, GdomeBoolean deep)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return NULL;
}

static GdomeDOMString *
gdome_xml_a_name (GdomeAttr *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return gdome_xml_str_mkref (a->name);
}

static GdomeBoolean
gdome_xml_a_specified (GdomeAttr *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;

  return 0;
}

static GdomeDOMString *
gdome_xml_a_value (GdomeAttr *self)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;
  xmlNode *val = a->val;

  /* todo: expand entity references; bleah! */
  return gdome_xml_str_mkref (val->content);
}

static void
gdome_xml_a_set_value (GdomeAttr *self, GdomeDOMString *value)
{
  Gdome_xml_Attr *priv = (Gdome_xml_Attr *)self;
  xmlAttr *a = priv->a;
  xmlNode *val = a->val;

  xmlNodeSetContent (val, value->str);
}

/* implementations of methods in interface CharacterData */

static GdomeDOMString *
gdome_xml_cd_data (GdomeCharacterData *self)
{
  Gdome_xml_CharacterData *priv = (Gdome_xml_CharacterData *)self;
  xmlNode *n = priv->n;

  return gdome_xml_str_mkref (n->content);
}

static void
gdome_xml_cd_set_data (GdomeCharacterData *self, GdomeDOMString *data)
{
  Gdome_xml_CharacterData *priv = (Gdome_xml_CharacterData *)self;
  xmlNode *n = priv->n;

  xmlNodeSetContent (n, data->str);
}

static unsigned long
gdome_xml_cd_length (GdomeCharacterData *self)
{
  Gdome_xml_CharacterData *priv = (Gdome_xml_CharacterData *)self;
  xmlNode *n = priv->n;
  CHAR c;
  CHAR *str = n->content;
  unsigned long i, len;

  /* utf-8 string length counting code */
  len = 0;
  for (i = 0; (c = str[i]) != '\0'; i++)
    if ((c & 0xc0) != 0x80)
      len++;
  return len;
}

static GdomeDOMString *
gdome_xml_cd_substringData (GdomeCharacterData *self, unsigned long offset, unsigned long count)
{
  Gdome_xml_CharacterData *priv = (Gdome_xml_CharacterData *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static void
gdome_xml_cd_appendData (GdomeCharacterData *self, GdomeDOMString *arg)
{
  Gdome_xml_CharacterData *priv = (Gdome_xml_CharacterData *)self;
  xmlNode *n = priv->n;

}

static void
gdome_xml_cd_insertData (GdomeCharacterData *self, unsigned long offset, GdomeDOMString *arg)
{
  Gdome_xml_CharacterData *priv = (Gdome_xml_CharacterData *)self;
  xmlNode *n = priv->n;

}

static void
gdome_xml_cd_deleteData (GdomeCharacterData *self, unsigned long offset, unsigned long count)
{
  Gdome_xml_CharacterData *priv = (Gdome_xml_CharacterData *)self;
  xmlNode *n = priv->n;

}

static void
gdome_xml_cd_replaceData (GdomeCharacterData *self, unsigned long offset, unsigned long count, GdomeDOMString *arg)
{
  Gdome_xml_CharacterData *priv = (Gdome_xml_CharacterData *)self;
  xmlNode *n = priv->n;

}

/* implementations of methods in interface Text */

static GdomeText *
gdome_xml_t_splitText (GdomeText *self, unsigned long offset)
{
  Gdome_xml_Text *priv = (Gdome_xml_Text *)self;
  xmlNode *n = priv->n;

  return NULL;
}

/* implementations of methods in interface CDATASection */

/* implementations of methods in interface Comment */

/* implementations of methods in interface DOMImplementation */

static void
gdome_xml_domi_unref (GdomeDOMImplementation *self)
{
  g_free (self);
}

static GdomeBoolean
gdome_xml_domi_hasFeature (GdomeDOMImplementation *self, GdomeDOMString *feature, GdomeDOMString *version)
{
  Gdome_xml_DOMImplementation *priv = (Gdome_xml_DOMImplementation *)self;

  return 0;
}

/* implementations of methods in interface Document */

static void
gdome_xml_doc_unref (GdomeNode *self)
{
  g_free (self);
}

static GdomeDOMString *
gdome_xml_doc_nodeName (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return gdome_xml_str_mkref ("#document");
}

static GdomeDOMString *
gdome_xml_doc_nodeValue (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static void
gdome_xml_doc_set_nodeValue (GdomeNode *self, GdomeDOMString *nodeValue)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

}

static unsigned short
gdome_xml_doc_nodeType (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return GDOME_DOCUMENT_NODE;
}

static GdomeNode *
gdome_xml_doc_parentNode (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNodeList *
gdome_xml_doc_childNodes (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNode *
gdome_xml_doc_firstChild (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNode *
gdome_xml_doc_lastChild (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNode *
gdome_xml_doc_previousSibling (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNode *
gdome_xml_doc_nextSibling (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNamedNodeMap *
gdome_xml_doc_attributes (GdomeNode *self)
{
  return NULL;
}

static GdomeDocument *
gdome_xml_doc_ownerDocument (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNode *
gdome_xml_doc_insertBefore (GdomeNode *self, GdomeNode *newChild, GdomeNode *refChild)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNode *
gdome_xml_doc_replaceChild (GdomeNode *self, GdomeNode *newChild, GdomeNode *oldChild)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNode *
gdome_xml_doc_removeChild (GdomeNode *self, GdomeNode *oldChild)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNode *
gdome_xml_doc_appendChild (GdomeNode *self, GdomeNode *newChild)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeBoolean
gdome_xml_doc_hasChildNodes (GdomeNode *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return 0;
}

static GdomeNode *
gdome_xml_doc_cloneNode (GdomeNode *self, GdomeBoolean deep)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeDocumentType *
gdome_xml_doc_doctype (GdomeDocument *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeDOMImplementation *
gdome_xml_doc_implementation (GdomeDocument *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeElement *
gdome_xml_doc_documentElement (GdomeDocument *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return (GdomeElement *)gdome_xml_n_mkref (doc->root);
}

static GdomeElement *
gdome_xml_doc_createElement (GdomeDocument *self, GdomeDOMString *tagName)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeDocumentFragment *
gdome_xml_doc_createDocumentFragment (GdomeDocument *self)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeText *
gdome_xml_doc_createTextNode (GdomeDocument *self, GdomeDOMString *data)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeComment *
gdome_xml_doc_createComment (GdomeDocument *self, GdomeDOMString *data)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeCDATASection *
gdome_xml_doc_createCDATASection (GdomeDocument *self, GdomeDOMString *data)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeProcessingInstruction *
gdome_xml_doc_createProcessingInstruction (GdomeDocument *self, GdomeDOMString *target, GdomeDOMString *data)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeAttr *
gdome_xml_doc_createAttribute (GdomeDocument *self, GdomeDOMString *name)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeEntityReference *
gdome_xml_doc_createEntityReference (GdomeDocument *self, GdomeDOMString *name)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

static GdomeNodeList *
gdome_xml_doc_getElementsByTagName (GdomeDocument *self, GdomeDOMString *tagname)
{
  Gdome_xml_Document *priv = (Gdome_xml_Document *)self;
  xmlDoc *doc = priv->doc;

  return NULL;
}

/* implementations of methods in interface DocumentFragment */

/* implementations of methods in interface DocumentType */

static void
gdome_xml_dt_unref (GdomeNode *self)
{
  g_free (self);
}

static GdomeDOMString *
gdome_xml_dt_nodeName (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeDOMString *
gdome_xml_dt_nodeValue (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static void
gdome_xml_dt_set_nodeValue (GdomeNode *self, GdomeDOMString *nodeValue)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

}

static unsigned short
gdome_xml_dt_nodeType (GdomeNode *self)
{
  return GDOME_DOCUMENT_TYPE_NODE;
}

static GdomeNode *
gdome_xml_dt_parentNode (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNodeList *
gdome_xml_dt_childNodes (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNode *
gdome_xml_dt_firstChild (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNode *
gdome_xml_dt_lastChild (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNode *
gdome_xml_dt_previousSibling (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNode *
gdome_xml_dt_nextSibling (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNamedNodeMap *
gdome_xml_dt_attributes (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeDocument *
gdome_xml_dt_ownerDocument (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNode *
gdome_xml_dt_insertBefore (GdomeNode *self, GdomeNode *newChild, GdomeNode *refChild)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNode *
gdome_xml_dt_replaceChild (GdomeNode *self, GdomeNode *newChild, GdomeNode *oldChild)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNode *
gdome_xml_dt_removeChild (GdomeNode *self, GdomeNode *oldChild)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNode *
gdome_xml_dt_appendChild (GdomeNode *self, GdomeNode *newChild)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeBoolean
gdome_xml_dt_hasChildNodes (GdomeNode *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return 0;
}

static GdomeNode *
gdome_xml_dt_cloneNode (GdomeNode *self, GdomeBoolean deep)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeDOMString *
gdome_xml_dt_name (GdomeDocumentType *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNamedNodeMap *
gdome_xml_dt_entities (GdomeDocumentType *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

static GdomeNamedNodeMap *
gdome_xml_dt_notations (GdomeDocumentType *self)
{
  Gdome_xml_DocumentType *priv = (Gdome_xml_DocumentType *)self;
  xmlDtd *dt = priv->dt;

  return NULL;
}

/* implementations of methods in interface Element */

static GdomeDOMString *
gdome_xml_el_tagName (GdomeElement *self)
{
  Gdome_xml_Element *priv = (Gdome_xml_Element *)self;
  xmlNode *n = priv->n;

  return gdome_xml_str_mkref (n->name);
}

static GdomeDOMString *
gdome_xml_el_getAttribute (GdomeElement *self, GdomeDOMString *name)
{
  Gdome_xml_Element *priv = (Gdome_xml_Element *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static void
gdome_xml_el_setAttribute (GdomeElement *self, GdomeDOMString *name, GdomeDOMString *value)
{
  Gdome_xml_Element *priv = (Gdome_xml_Element *)self;
  xmlNode *n = priv->n;

}

static void
gdome_xml_el_removeAttribute (GdomeElement *self, GdomeDOMString *name)
{
  Gdome_xml_Element *priv = (Gdome_xml_Element *)self;
  xmlNode *n = priv->n;

}

static GdomeAttr *
gdome_xml_el_getAttributeNode (GdomeElement *self, GdomeDOMString *name)
{
  Gdome_xml_Element *priv = (Gdome_xml_Element *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeAttr *
gdome_xml_el_setAttributeNode (GdomeElement *self, GdomeAttr *newAttr)
{
  Gdome_xml_Element *priv = (Gdome_xml_Element *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeAttr *
gdome_xml_el_removeAttributeNode (GdomeElement *self, GdomeAttr *oldAttr)
{
  Gdome_xml_Element *priv = (Gdome_xml_Element *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeNodeList *
gdome_xml_el_getElementsByTagName (GdomeElement *self, GdomeDOMString *name)
{
  Gdome_xml_Element *priv = (Gdome_xml_Element *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static void
gdome_xml_el_normalize (GdomeElement *self)
{
  Gdome_xml_Element *priv = (Gdome_xml_Element *)self;
  xmlNode *n = priv->n;

}

/* implementations of methods in interface Entity */

static GdomeDOMString *
gdome_xml_ent_publicId (GdomeEntity *self)
{
  Gdome_xml_Entity *priv = (Gdome_xml_Entity *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeDOMString *
gdome_xml_ent_systemId (GdomeEntity *self)
{
  Gdome_xml_Entity *priv = (Gdome_xml_Entity *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeDOMString *
gdome_xml_ent_notationName (GdomeEntity *self)
{
  Gdome_xml_Entity *priv = (Gdome_xml_Entity *)self;
  xmlNode *n = priv->n;

  return NULL;
}

/* implementations of methods in interface EntityReference */

/* implementations of methods in interface NamedNodeMap */

static void
gdome_xml_nnm_unref (GdomeNamedNodeMap *self)
{
  Gdome_xml_NamedNodeMap *priv = (Gdome_xml_NamedNodeMap *)self;

}

static GdomeNode *
gdome_xml_nnm_getNamedItem (GdomeNamedNodeMap *self, GdomeDOMString *name)
{
  Gdome_xml_NamedNodeMap *priv = (Gdome_xml_NamedNodeMap *)self;

  return NULL;
}

static GdomeNode *
gdome_xml_nnm_setNamedItem (GdomeNamedNodeMap *self, GdomeNode *arg)
{
  Gdome_xml_NamedNodeMap *priv = (Gdome_xml_NamedNodeMap *)self;

  return NULL;
}

static GdomeNode *
gdome_xml_nnm_removeNamedItem (GdomeNamedNodeMap *self, GdomeDOMString *name)
{
  Gdome_xml_NamedNodeMap *priv = (Gdome_xml_NamedNodeMap *)self;

  return NULL;
}

static GdomeNode *
gdome_xml_nnm_item (GdomeNamedNodeMap *self, unsigned long index)
{
  Gdome_xml_NamedNodeMap *priv = (Gdome_xml_NamedNodeMap *)self;

  return NULL;
}

static unsigned long
gdome_xml_nnm_length (GdomeNamedNodeMap *self)
{
  Gdome_xml_NamedNodeMap *priv = (Gdome_xml_NamedNodeMap *)self;

  return 0;
}

/* implementations of methods in interface NamedNodeMap,
   specialized to the implementation of the attributes method on Element */

static void
gdome_xml_el_nnm_unref (GdomeNamedNodeMap *self)
{
  g_free (self);
}

static GdomeNode *
gdome_xml_el_nnm_getNamedItem (GdomeNamedNodeMap *self, GdomeDOMString *name)
{
  Gdome_xml_el_NamedNodeMap *priv = (Gdome_xml_el_NamedNodeMap *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeNode *
gdome_xml_el_nnm_setNamedItem (GdomeNamedNodeMap *self, GdomeNode *arg)
{
  Gdome_xml_el_NamedNodeMap *priv = (Gdome_xml_el_NamedNodeMap *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeNode *
gdome_xml_el_nnm_removeNamedItem (GdomeNamedNodeMap *self, GdomeDOMString *name)
{
  Gdome_xml_el_NamedNodeMap *priv = (Gdome_xml_el_NamedNodeMap *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeNode *
gdome_xml_el_nnm_item (GdomeNamedNodeMap *self, unsigned long index)
{
  Gdome_xml_el_NamedNodeMap *priv = (Gdome_xml_el_NamedNodeMap *)self;
  xmlNode *n = priv->n;
  xmlAttr *a = n->properties;
  unsigned long i;
  Gdome_xml_Attr *result;

  for (i = 0; a != NULL && i < index; i++)
    a = a->next;
  if (a == NULL)
    return NULL;
  result = g_new (Gdome_xml_Attr, 1);
  result->super.vtab = (GdomeAttrVtab *)&gdome_xml_a_vtab;
  result->a = a;
  return (GdomeNode *)result;
}

static unsigned long
gdome_xml_el_nnm_length (GdomeNamedNodeMap *self)
{
  Gdome_xml_el_NamedNodeMap *priv = (Gdome_xml_el_NamedNodeMap *)self;
  xmlNode *n = priv->n;
  xmlAttr *a = n->properties;
  unsigned long i;

  for (i = 0; a != NULL; i++)
    a = a->next;
  return i;
}

/* implementations of methods in interface NodeList */

static void
gdome_xml_nl_unref (GdomeNodeList *self)
{
  Gdome_xml_NodeList *priv = (Gdome_xml_NodeList *)self;

}

static GdomeNode *
gdome_xml_nl_item (GdomeNodeList *self, unsigned long index)
{
  Gdome_xml_NodeList *priv = (Gdome_xml_NodeList *)self;

  return NULL;
}

static unsigned long
gdome_xml_nl_length (GdomeNodeList *self)
{
  Gdome_xml_NodeList *priv = (Gdome_xml_NodeList *)self;

  return 0;
}

/* implementations of methods in interface Notation */

static void
gdome_xml_not_unref (GdomeNode *self)
{
  g_free (self);
}

static GdomeDOMString *
gdome_xml_not_nodeName (GdomeNode *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return gdome_xml_str_mkref ((CHAR *)not->name);
}

static GdomeDOMString *
gdome_xml_not_nodeValue (GdomeNode *self)
{
  return NULL;
}

static void
gdome_xml_not_set_nodeValue (GdomeNode *self, GdomeDOMString *nodeValue)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

}

static unsigned short
gdome_xml_not_nodeType (GdomeNode *self)
{
  return GDOME_NOTATION_NODE;
}

static GdomeNode *
gdome_xml_not_parentNode (GdomeNode *self)
{
  return NULL;
}

static GdomeNodeList *
gdome_xml_not_childNodes (GdomeNode *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeNode *
gdome_xml_not_firstChild (GdomeNode *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeNode *
gdome_xml_not_lastChild (GdomeNode *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeNode *
gdome_xml_not_previousSibling (GdomeNode *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeNode *
gdome_xml_not_nextSibling (GdomeNode *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeNamedNodeMap *
gdome_xml_not_attributes (GdomeNode *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeDocument *
gdome_xml_not_ownerDocument (GdomeNode *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeNode *
gdome_xml_not_insertBefore (GdomeNode *self, GdomeNode *newChild, GdomeNode *refChild)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeNode *
gdome_xml_not_replaceChild (GdomeNode *self, GdomeNode *newChild, GdomeNode *oldChild)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeNode *
gdome_xml_not_removeChild (GdomeNode *self, GdomeNode *oldChild)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeNode *
gdome_xml_not_appendChild (GdomeNode *self, GdomeNode *newChild)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeBoolean
gdome_xml_not_hasChildNodes (GdomeNode *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return 0;
}

static GdomeNode *
gdome_xml_not_cloneNode (GdomeNode *self, GdomeBoolean deep)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return NULL;
}

static GdomeDOMString *
gdome_xml_not_publicId (GdomeNotation *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return gdome_xml_str_mkref (not->PublicID);
}

static GdomeDOMString *
gdome_xml_not_systemId (GdomeNotation *self)
{
  Gdome_xml_Notation *priv = (Gdome_xml_Notation *)self;
  xmlNotation *not = priv->not;

  return gdome_xml_str_mkref (not->SystemID);
}

/* implementations of methods in interface Parser */

/* this is actually a nonstandard extension */

static void
gdome_xml_pars_unref (GdomeParser *self)
{
  Gdome_xml_Parser *priv = (Gdome_xml_Parser *)self;

}

static GdomeDocument *
gdome_xml_pars_xmlParseFile (GdomeParser *self, GdomeDOMString *filename)
{
  Gdome_xml_Parser *priv = (Gdome_xml_Parser *)self;

  return NULL;
}

static GdomeDocument *
gdome_xml_pars_xmlParseDoc (GdomeParser *self, GdomeDOMString *content)
{
  Gdome_xml_Parser *priv = (Gdome_xml_Parser *)self;

  return NULL;
}

/* implementations of methods in interface ProcessingInstruction */

static GdomeDOMString *
gdome_xml_pi_target (GdomeProcessingInstruction *self)
{
  Gdome_xml_ProcessingInstruction *priv = (Gdome_xml_ProcessingInstruction *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static GdomeDOMString *
gdome_xml_pi_data (GdomeProcessingInstruction *self)
{
  Gdome_xml_ProcessingInstruction *priv = (Gdome_xml_ProcessingInstruction *)self;
  xmlNode *n = priv->n;

  return NULL;
}

static void
gdome_xml_pi_set_data (GdomeProcessingInstruction *self, GdomeDOMString *data)
{
  Gdome_xml_ProcessingInstruction *priv = (Gdome_xml_ProcessingInstruction *)self;
  xmlNode *n = priv->n;

}

const GdomeNodeVtab gdome_xml_n_vtab = {
  gdome_xml_n_unref,
  gdome_xml_n_nodeName,
  gdome_xml_n_nodeValue,
  gdome_xml_n_set_nodeValue,
  gdome_xml_n_nodeType,
  gdome_xml_n_parentNode,
  gdome_xml_n_childNodes,
  gdome_xml_n_firstChild,
  gdome_xml_n_lastChild,
  gdome_xml_n_previousSibling,
  gdome_xml_n_nextSibling,
  gdome_xml_n_attributes,
  gdome_xml_n_ownerDocument,
  gdome_xml_n_insertBefore,
  gdome_xml_n_replaceChild,
  gdome_xml_n_removeChild,
  gdome_xml_n_appendChild,
  gdome_xml_n_hasChildNodes,
  gdome_xml_n_cloneNode
};

const GdomeAttrVtab gdome_xml_a_vtab = {
  {
    gdome_xml_a_unref,
    gdome_xml_a_nodeName,
    gdome_xml_a_nodeValue,
    gdome_xml_a_set_nodeValue,
    gdome_xml_a_nodeType,
    gdome_xml_a_parentNode,
    gdome_xml_a_childNodes,
    gdome_xml_a_firstChild,
    gdome_xml_a_lastChild,
    gdome_xml_a_previousSibling,
    gdome_xml_a_nextSibling,
    gdome_xml_a_attributes,
    gdome_xml_a_ownerDocument,
    gdome_xml_a_insertBefore,
    gdome_xml_a_replaceChild,
    gdome_xml_a_removeChild,
    gdome_xml_a_appendChild,
    gdome_xml_a_hasChildNodes,
    gdome_xml_a_cloneNode
  },
  gdome_xml_a_name,
  gdome_xml_a_specified,
  gdome_xml_a_value,
  gdome_xml_a_set_value
};

const GdomeCharacterDataVtab gdome_xml_cd_vtab = {
  {
    gdome_xml_n_unref,
    gdome_xml_n_nodeName,
    gdome_xml_n_nodeValue,
    gdome_xml_n_set_nodeValue,
    gdome_xml_n_nodeType,
    gdome_xml_n_parentNode,
    gdome_xml_n_childNodes,
    gdome_xml_n_firstChild,
    gdome_xml_n_lastChild,
    gdome_xml_n_previousSibling,
    gdome_xml_n_nextSibling,
    gdome_xml_n_attributes,
    gdome_xml_n_ownerDocument,
    gdome_xml_n_insertBefore,
    gdome_xml_n_replaceChild,
    gdome_xml_n_removeChild,
    gdome_xml_n_appendChild,
    gdome_xml_n_hasChildNodes,
    gdome_xml_n_cloneNode
  },
  gdome_xml_cd_data,
  gdome_xml_cd_set_data,
  gdome_xml_cd_length,
  gdome_xml_cd_substringData,
  gdome_xml_cd_appendData,
  gdome_xml_cd_insertData,
  gdome_xml_cd_deleteData,
  gdome_xml_cd_replaceData
};

const GdomeTextVtab gdome_xml_t_vtab = {
  {
    {
      gdome_xml_n_unref,
      gdome_xml_n_nodeName,
      gdome_xml_n_nodeValue,
      gdome_xml_n_set_nodeValue,
      gdome_xml_n_nodeType,
      gdome_xml_n_parentNode,
      gdome_xml_n_childNodes,
      gdome_xml_n_firstChild,
      gdome_xml_n_lastChild,
      gdome_xml_n_previousSibling,
      gdome_xml_n_nextSibling,
      gdome_xml_n_attributes,
      gdome_xml_n_ownerDocument,
      gdome_xml_n_insertBefore,
      gdome_xml_n_replaceChild,
      gdome_xml_n_removeChild,
      gdome_xml_n_appendChild,
      gdome_xml_n_hasChildNodes,
      gdome_xml_n_cloneNode
    },
    gdome_xml_cd_data,
    gdome_xml_cd_set_data,
    gdome_xml_cd_length,
    gdome_xml_cd_substringData,
    gdome_xml_cd_appendData,
    gdome_xml_cd_insertData,
    gdome_xml_cd_deleteData,
    gdome_xml_cd_replaceData
  },
  gdome_xml_t_splitText
};

const GdomeCDATASectionVtab gdome_xml_cds_vtab = {
  {
    {
      {
        gdome_xml_n_unref,
        gdome_xml_n_nodeName,
        gdome_xml_n_nodeValue,
        gdome_xml_n_set_nodeValue,
        gdome_xml_n_nodeType,
        gdome_xml_n_parentNode,
        gdome_xml_n_childNodes,
        gdome_xml_n_firstChild,
        gdome_xml_n_lastChild,
        gdome_xml_n_previousSibling,
        gdome_xml_n_nextSibling,
        gdome_xml_n_attributes,
        gdome_xml_n_ownerDocument,
        gdome_xml_n_insertBefore,
        gdome_xml_n_replaceChild,
        gdome_xml_n_removeChild,
        gdome_xml_n_appendChild,
        gdome_xml_n_hasChildNodes,
        gdome_xml_n_cloneNode
      },
      gdome_xml_cd_data,
      gdome_xml_cd_set_data,
      gdome_xml_cd_length,
      gdome_xml_cd_substringData,
      gdome_xml_cd_appendData,
      gdome_xml_cd_insertData,
      gdome_xml_cd_deleteData,
      gdome_xml_cd_replaceData
    },
    gdome_xml_t_splitText
  }
};

const GdomeCommentVtab gdome_xml_c_vtab = {
  {
    {
      gdome_xml_n_unref,
      gdome_xml_n_nodeName,
      gdome_xml_n_nodeValue,
      gdome_xml_n_set_nodeValue,
      gdome_xml_n_nodeType,
      gdome_xml_n_parentNode,
      gdome_xml_n_childNodes,
      gdome_xml_n_firstChild,
      gdome_xml_n_lastChild,
      gdome_xml_n_previousSibling,
      gdome_xml_n_nextSibling,
      gdome_xml_n_attributes,
      gdome_xml_n_ownerDocument,
      gdome_xml_n_insertBefore,
      gdome_xml_n_replaceChild,
      gdome_xml_n_removeChild,
      gdome_xml_n_appendChild,
      gdome_xml_n_hasChildNodes,
      gdome_xml_n_cloneNode
    },
    gdome_xml_cd_data,
    gdome_xml_cd_set_data,
    gdome_xml_cd_length,
    gdome_xml_cd_substringData,
    gdome_xml_cd_appendData,
    gdome_xml_cd_insertData,
    gdome_xml_cd_deleteData,
    gdome_xml_cd_replaceData
  }
};

const GdomeDOMImplementationVtab gdome_xml_domi_vtab = {
  gdome_xml_domi_unref,
  gdome_xml_domi_hasFeature
};

const GdomeDocumentVtab gdome_xml_doc_vtab = {
  {
    gdome_xml_doc_unref,
    gdome_xml_doc_nodeName,
    gdome_xml_doc_nodeValue,
    gdome_xml_doc_set_nodeValue,
    gdome_xml_doc_nodeType,
    gdome_xml_doc_parentNode,
    gdome_xml_doc_childNodes,
    gdome_xml_doc_firstChild,
    gdome_xml_doc_lastChild,
    gdome_xml_doc_previousSibling,
    gdome_xml_doc_nextSibling,
    gdome_xml_doc_attributes,
    gdome_xml_doc_ownerDocument,
    gdome_xml_doc_insertBefore,
    gdome_xml_doc_replaceChild,
    gdome_xml_doc_removeChild,
    gdome_xml_doc_appendChild,
    gdome_xml_doc_hasChildNodes,
    gdome_xml_doc_cloneNode
  },
  gdome_xml_doc_doctype,
  gdome_xml_doc_implementation,
  gdome_xml_doc_documentElement,
  gdome_xml_doc_createElement,
  gdome_xml_doc_createDocumentFragment,
  gdome_xml_doc_createTextNode,
  gdome_xml_doc_createComment,
  gdome_xml_doc_createCDATASection,
  gdome_xml_doc_createProcessingInstruction,
  gdome_xml_doc_createAttribute,
  gdome_xml_doc_createEntityReference,
  gdome_xml_doc_getElementsByTagName
};

const GdomeDocumentFragmentVtab gdome_xml_df_vtab = {
  {
    gdome_xml_n_unref,
    gdome_xml_n_nodeName,
    gdome_xml_n_nodeValue,
    gdome_xml_n_set_nodeValue,
    gdome_xml_n_nodeType,
    gdome_xml_n_parentNode,
    gdome_xml_n_childNodes,
    gdome_xml_n_firstChild,
    gdome_xml_n_lastChild,
    gdome_xml_n_previousSibling,
    gdome_xml_n_nextSibling,
    gdome_xml_n_attributes,
    gdome_xml_n_ownerDocument,
    gdome_xml_n_insertBefore,
    gdome_xml_n_replaceChild,
    gdome_xml_n_removeChild,
    gdome_xml_n_appendChild,
    gdome_xml_n_hasChildNodes,
    gdome_xml_n_cloneNode
  }
};

const GdomeDocumentTypeVtab gdome_xml_dt_vtab = {
  {
    gdome_xml_dt_unref,
    gdome_xml_dt_nodeName,
    gdome_xml_dt_nodeValue,
    gdome_xml_dt_set_nodeValue,
    gdome_xml_dt_nodeType,
    gdome_xml_dt_parentNode,
    gdome_xml_dt_childNodes,
    gdome_xml_dt_firstChild,
    gdome_xml_dt_lastChild,
    gdome_xml_dt_previousSibling,
    gdome_xml_dt_nextSibling,
    gdome_xml_dt_attributes,
    gdome_xml_dt_ownerDocument,
    gdome_xml_dt_insertBefore,
    gdome_xml_dt_replaceChild,
    gdome_xml_dt_removeChild,
    gdome_xml_dt_appendChild,
    gdome_xml_dt_hasChildNodes,
    gdome_xml_dt_cloneNode
  },
  gdome_xml_dt_name,
  gdome_xml_dt_entities,
  gdome_xml_dt_notations
};

const GdomeElementVtab gdome_xml_el_vtab = {
  {
    gdome_xml_n_unref,
    gdome_xml_n_nodeName,
    gdome_xml_n_nodeValue,
    gdome_xml_n_set_nodeValue,
    gdome_xml_n_nodeType,
    gdome_xml_n_parentNode,
    gdome_xml_n_childNodes,
    gdome_xml_n_firstChild,
    gdome_xml_n_lastChild,
    gdome_xml_n_previousSibling,
    gdome_xml_n_nextSibling,
    gdome_xml_n_attributes,
    gdome_xml_n_ownerDocument,
    gdome_xml_n_insertBefore,
    gdome_xml_n_replaceChild,
    gdome_xml_n_removeChild,
    gdome_xml_n_appendChild,
    gdome_xml_n_hasChildNodes,
    gdome_xml_n_cloneNode
  },
  gdome_xml_el_tagName,
  gdome_xml_el_getAttribute,
  gdome_xml_el_setAttribute,
  gdome_xml_el_removeAttribute,
  gdome_xml_el_getAttributeNode,
  gdome_xml_el_setAttributeNode,
  gdome_xml_el_removeAttributeNode,
  gdome_xml_el_getElementsByTagName,
  gdome_xml_el_normalize
};

const GdomeEntityVtab gdome_xml_ent_vtab = {
  {
    gdome_xml_n_unref,
    gdome_xml_n_nodeName,
    gdome_xml_n_nodeValue,
    gdome_xml_n_set_nodeValue,
    gdome_xml_n_nodeType,
    gdome_xml_n_parentNode,
    gdome_xml_n_childNodes,
    gdome_xml_n_firstChild,
    gdome_xml_n_lastChild,
    gdome_xml_n_previousSibling,
    gdome_xml_n_nextSibling,
    gdome_xml_n_attributes,
    gdome_xml_n_ownerDocument,
    gdome_xml_n_insertBefore,
    gdome_xml_n_replaceChild,
    gdome_xml_n_removeChild,
    gdome_xml_n_appendChild,
    gdome_xml_n_hasChildNodes,
    gdome_xml_n_cloneNode
  },
  gdome_xml_ent_publicId,
  gdome_xml_ent_systemId,
  gdome_xml_ent_notationName
};

const GdomeEntityReferenceVtab gdome_xml_er_vtab = {
  {
    gdome_xml_n_unref,
    gdome_xml_n_nodeName,
    gdome_xml_n_nodeValue,
    gdome_xml_n_set_nodeValue,
    gdome_xml_n_nodeType,
    gdome_xml_n_parentNode,
    gdome_xml_n_childNodes,
    gdome_xml_n_firstChild,
    gdome_xml_n_lastChild,
    gdome_xml_n_previousSibling,
    gdome_xml_n_nextSibling,
    gdome_xml_n_attributes,
    gdome_xml_n_ownerDocument,
    gdome_xml_n_insertBefore,
    gdome_xml_n_replaceChild,
    gdome_xml_n_removeChild,
    gdome_xml_n_appendChild,
    gdome_xml_n_hasChildNodes,
    gdome_xml_n_cloneNode
  }
};

const GdomeNamedNodeMapVtab gdome_xml_nnm_vtab = {
  gdome_xml_nnm_unref,
  gdome_xml_nnm_getNamedItem,
  gdome_xml_nnm_setNamedItem,
  gdome_xml_nnm_removeNamedItem,
  gdome_xml_nnm_item,
  gdome_xml_nnm_length
};

const GdomeNamedNodeMapVtab gdome_xml_el_nnm_vtab = {
  gdome_xml_el_nnm_unref,
  gdome_xml_el_nnm_getNamedItem,
  gdome_xml_el_nnm_setNamedItem,
  gdome_xml_el_nnm_removeNamedItem,
  gdome_xml_el_nnm_item,
  gdome_xml_el_nnm_length
};

const GdomeNodeListVtab gdome_xml_nl_vtab = {
  gdome_xml_nl_unref,
  gdome_xml_nl_item,
  gdome_xml_nl_length
};

const GdomeNotationVtab gdome_xml_not_vtab = {
  {
    gdome_xml_not_unref,
    gdome_xml_not_nodeName,
    gdome_xml_not_nodeValue,
    gdome_xml_not_set_nodeValue,
    gdome_xml_not_nodeType,
    gdome_xml_not_parentNode,
    gdome_xml_not_childNodes,
    gdome_xml_not_firstChild,
    gdome_xml_not_lastChild,
    gdome_xml_not_previousSibling,
    gdome_xml_not_nextSibling,
    gdome_xml_not_attributes,
    gdome_xml_not_ownerDocument,
    gdome_xml_not_insertBefore,
    gdome_xml_not_replaceChild,
    gdome_xml_not_removeChild,
    gdome_xml_not_appendChild,
    gdome_xml_not_hasChildNodes,
    gdome_xml_not_cloneNode
  },
  gdome_xml_not_publicId,
  gdome_xml_not_systemId
};

const GdomeParserVtab gdome_xml_pars_vtab = {
  gdome_xml_pars_unref,
  gdome_xml_pars_xmlParseFile,
  gdome_xml_pars_xmlParseDoc
};

const GdomeProcessingInstructionVtab gdome_xml_pi_vtab = {
  {
    gdome_xml_n_unref,
    gdome_xml_n_nodeName,
    gdome_xml_n_nodeValue,
    gdome_xml_n_set_nodeValue,
    gdome_xml_n_nodeType,
    gdome_xml_n_parentNode,
    gdome_xml_n_childNodes,
    gdome_xml_n_firstChild,
    gdome_xml_n_lastChild,
    gdome_xml_n_previousSibling,
    gdome_xml_n_nextSibling,
    gdome_xml_n_attributes,
    gdome_xml_n_ownerDocument,
    gdome_xml_n_insertBefore,
    gdome_xml_n_replaceChild,
    gdome_xml_n_removeChild,
    gdome_xml_n_appendChild,
    gdome_xml_n_hasChildNodes,
    gdome_xml_n_cloneNode
  },
  gdome_xml_pi_target,
  gdome_xml_pi_data,
  gdome_xml_pi_set_data
};

GdomeDocument *
gdome_xml_from_document (xmlDoc *doc)
{
  Gdome_xml_Document *result;

  result = g_new (Gdome_xml_Document, 1);
  result->super.vtab = (GdomeDocumentVtab *)&gdome_xml_doc_vtab;
  result->doc = doc;

  return (GdomeDocument *)result;
}
