#include <stdio.h>
#include "gnome-xml/parser.h"
#include "gnome-xml/tree.h"
#include "gdome.h"
#include "gdome-xml.h"

void
print_gdoc_structure (GdomeNode *node)
{
  int type;
  GdomeNode *child, *next;
  GdomeDOMString *tagName;

  type = gdome_n_nodeType (node);
  if (type == GDOME_TEXT_NODE)
    {
      GdomeDOMString *text;

      text = gdome_cd_data ((GdomeCharacterData *)node);
      fputs (text->str, stdout);
      gdome_str_unref (text);

      return;
    }
  tagName = NULL;
  if (type == GDOME_ELEMENT_NODE)
    {
      GdomeNamedNodeMap *attrs;
      int i, n_attrs;
      putchar ('<');
      tagName = gdome_el_tagName ((GdomeElement *)node);
      fputs (tagName->str, stdout);

      attrs = gdome_n_attributes (node);
      n_attrs = gdome_nnm_length (attrs);
      for (i = 0; i < n_attrs; i++)
	{
	  GdomeAttr *attr;
	  GdomeDOMString *attrName, *attrVal;
	  attr = gdome_nnm_item (attrs, i);
	  attrName = gdome_a_name (attr);
	  attrVal = gdome_a_value (attr);
	  gdome_n_unref (attr);
	  printf (" %s=\"%s\"", attrName->str, attrVal->str);
	  gdome_str_unref (attrName);
	  gdome_str_unref (attrVal);
	}
      gdome_n_unref (attrs);
      putchar ('>');
    }
  for (child = gdome_n_firstChild (node); child != NULL; child = next)
    {
      print_gdoc_structure (child);
      next = gdome_n_nextSibling (child);
      gdome_n_unref (child);
    }
  if (type == GDOME_ELEMENT_NODE)
    {
      fputs ("</", stdout);
      fputs (tagName->str, stdout);
      gdome_str_unref (tagName);
      putchar ('>');
    }
}

int
main (int argc, char **argv)
{
  xmlDocPtr doc;
  GdomeDocument *gdoc;
  GdomeElement *el;

  if (argc < 2)
    {
      fprintf (stderr, "usage: test-gdome test.xml\n");
      exit (1);
    }

  doc = xmlParseFile (argv[1]);
  gdoc = gdome_xml_from_document (doc);
  el = gdome_doc_documentElement (gdoc);
  gdome_n_unref (gdoc);
  print_gdoc_structure ((GdomeNode *)el);
  gdome_n_unref (el);
  putchar ('\n');

  xmlFreeDoc (doc);
  return 0;
}
